#! perl

# --8<--8<--8<--8<--
#
# Copyright (C) 2011 Smithsonian Astrophysical Observatory
#
# This file is part of write-ecf-fits
#
# write-ecf-fits is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

use 5.10.0;

use strict;
use warnings;

package WECF::Data;

our $VERSION = '2.0.0';

use Moo;

use Params::Validate qw[ validate SCALAR ];

has data => ( is => 'ro',
	      init_arg => undef,
	      default => sub { {} },
	    );

my %spec = map {
    $_ => { type => SCALAR, optional => 0 }
}  qw[ energy theta phi ] ;

sub get {

    my $self = shift;

    my %arg = validate( @_, { name => { optional => 1 },
			      raise => { default => 0 },
			      %spec
			    } );

    my $data = my $ecf = $self->data->{$arg{energy}}{$arg{theta}}{$arg{phi}};

    $data = $ecf->data->{$arg{name}} if defined $ecf && defined $arg{name};

    if ( $arg{raise} && !defined $data )
    {
	delete $arg{raise};

	die( 'unknown data set: ',
	     join( ', ', map { "$_ = $arg{$_}"} keys %arg),
	     "\n"
	   );
    }

    return $data;
}

sub set {

    my $self = shift;
    my %arg = validate( @_, { ecf => { isa => 'WECF' }, %spec });

    $self->data->{$arg{energy}}{$arg{theta}}{$arg{phi}} = $arg{ecf};
}

sub resolve {

    my $self = shift;

    my %arg = validate( @_, { name => { optional => 0 },
			      %spec
			    } );

    my $datum = $self->get( %arg );

    return if ! defined $datum;

    if ( $datum->linked )
    {

	my $link = $self->get( $datum->link, name => $datum->name );
	return $link->resolve( $datum );
    }

    else
    {
	return $datum->value;
    }
}

1;
