#! perl

# --8<--8<--8<--8<--
#
# Copyright (C) 2011 Smithsonian Astrophysical Observatory
#
# This file is part of write-ecf-fits
#
# write-ecf-fits is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

use 5.10.0;

use strict;
use warnings;

package WECF::Datum;

our $VERSION = '2.0.0';

use Moo;

has name => ( is => 'ro',
	      required => 1,
	    );

has data => ( is => 'ro',
	      required=> 1,
	      clearer => '_clear_data'
	    );

has is_linked_to => ( is => 'ro',
		      required => 1 );

has link => ( is => 'ro',
	      isa => sub { 'HASH' eq ref $_[0] },
	      default => sub { {} }
	    );

around link => sub {
    my $orig = shift;
    my $self = shift;
    my $link = $orig->( $self, @_ );
    return wantarray ? %$link : $link;
};

has linked => ( is => 'ro',
		init_arg => undef,
		lazy => 1,
		builder => '_build_linked'
	      );

has value => ( is => 'ro',
	       writer => '_value',
	       init_arg => undef,
	     );

sub resolve { $_[0]->value }

sub _build_linked { 0 != keys %{$_[0]->link} }

# set value during object instantiation so can be
# overridden by child classes which wish to delete
# piddles needed only for its calculation. keeps
# memory usage down during construction of ECF objects
# before writing them out

sub BUILD {

    my $self = shift;

    $self->_value( $self->data );
}


1;
