#! perl

use strict;
use warnings;

use RDB;
use PDL::IO::RDB qw[ rdrdb ];
use PDL;
use PDL::Types;
use Astro::FITS::CFITSIO::Simple qw[ rdfits ];
use File::Spec::Functions;
use File::Find;

use File::Temp;

use lib 't';
require 'common.pl';

use Util::Any -list => [ 'mapp' ];

use Test::More tests => 48;

use WECF::Options;

use lib 'src';
require 'write-ecf-fits';

my $datadir = catdir( qw[ t data ] );

use vars qw[ $opt ];

my $if = catfile( $datadir, 'dr01.rdb');

my %ecf = make_fits();

my ( $fraction, $pars, $data ) =  xread_data();

check_output();

sub make_fits {

    my $tdir = 't';

    my $fits = File::Temp->new( DIR => $tdir, SUFFIX => '.fits' );

    my @args = ( '--output', $fits,
		 '--output_params',  $if,
		 qw[
		       --detector hrc-s
		       --config none
		       --frac ecf
		  ],
		 '--par', 'x,mm,x,0.1,0.1',
		 '--par', 'y,mm,y,0.1,0.1',
		 map { catdir( $tdir, $_ ) }
		 qw[
		       data/ecf/06.00/03.00/200.0/ell.rdb
		       data/ecf/08.00/04.00/116.0/ell.rdb
		       data/ecf/01.00/06.00/340.0/ell.rdb
		       data/ecf/01.00/02.00/329.0/ell.rdb
		  ],
	       );

    system( catfile( qw[ blib script write-ecf-fits] ), @args ) == 0
      or die( "error creating test fits file\n" );

    mapp { $a => $b->squeeze } rdfits( "$fits",
				       qw[ -energ_lo -energ_hi -ecf ],
				     {
				      defdtype => double }
				     );


}

sub check_output {

    my $rdb = RDB->new( $if ) or die( "unable to read/parse $if\n" );

    my %row;
    my @params = qw[ energy theta phi ];
    my @fparams = qw[ theta phi energy ];
    my %fmt = ( energy => '%05.2f',
		theta => '%05.2f',
		phi  => '%05.1f',
	      );

    my %ecfparam = map { $_ => delete $ecf{$_} } @params;

    while ( $rdb->read( \%row ) )
    {
	my %idx = map { $_ => vsearch ( $row{$_}, $ecfparam{$_} ) } @params;

	my @tags = map { sprintf( $fmt{$_}, $row{$_} ) } @params;
	my $tag = join( ',', @tags );

	my $df = catfile( $datadir, 'ecf', @tags, 'ell.rdb' );

	my %fid = mapp{
	    $a => $b,
	    "${a}_smin" => $b * 0.9,
	    "${a}_smax" => $b * 1.1,
	} read_fiducial( $df );

	$_ += 0 for values %row;

	for my $par ( keys %ecf ) {

	    my $ed = $ecf{$par}->mslice( [], map { [ $idx{$_} ] } @fparams )->squeeze;

	    ok( all( $ed == $data->resolve( %row, name => $par ) ), "$tag: $par" );

	    ok( all( $ed == $fid{$par} ), "$tag: $par" );
	}

    }

}

sub xread_data {

    my @params = qw [ x y sma smb pa ];
    my @files;
    find( sub { push @files, $File::Find::name if $_ eq 'ell.rdb' },
	  catdir( $datadir, 'ecf' ) );


    my @argv = ( qw[ --output foo.fits
		     --detector hrc-s
		     --config none
		     --frac ecf
		  ],
	       );

    my @pars = map { ( '--par' => "$_,${_}_units,$_,0.1,0.1" ) } @params;

    @ARGV = ( @argv, @pars, '--output_params', $if, @files );

    $opt = WECF::Options->new;

    my ( $fraction, $pars, $data ) =  read_data();

}
