#! perl

use strict;
use warnings;

use PDL::IO::Misc;
use PDL::IO::RDB qw[ rdrdb ];
use File::Spec::Functions;
use Test::More tests => 8;

use WECF::Options;
use WECF;
use WECF::OutputParams;

use lib 'src';
require 'write-ecf-fits';

use lib 't';
require 'common.pl';

use vars qw[ $opt ];

my $datadir = catdir( qw[ t data ] );

my %params = ( energy => '01.00',
	       theta  => '02.00',
	       phi    => '329.0'
	     );
my @params = @params{qw [energy theta phi ]};
my @pars = qw [ x y sma smb pa ];

my $file = catfile( $datadir, 'ecf', @params, 'ell.rdb' );
my $if = catfile( $datadir, 'dr01.rdb');

@ARGV = ( qw[ --output foo.fits
              --detector hrc-s
              --config none
              --frac ecf
            ],
	  '--output_params', $if,
	  (
	    map { ( '--par' => "$_,${_}_units,$_" ) } @pars
	  ),
	  $file
    );

$opt = WECF::Options->new;
my $opars = WECF::OutputParams->new( file => $if );

my %fparam = params_from_filename( $opt->re, $file );
my $ecf = WECF->new( file => $file,
		    param => \%fparam,
		    verbose => $opt->verbose,
		    frac => $opt->frac,
		    epars => $opt->pars,
		    opars => $opars
		  );

is ( $ecf->energy, $params{energy}+0, 'energy' );
is ( $ecf->theta,  $params{theta}+0,  'theta' );
is ( $ecf->phi  ,  $params{phi}+0,    'phi' );

# ensure that the data were correctly read in
my %rdb = read_fiducial( $file );

for my $col ( @pars ) {

    ok( all ( $rdb{$col} == $ecf->data->{$col}->value ), "$col" );

}

